package convert.buffers;

import java.util.concurrent.CountDownLatch;

import org.apache.commons.collections.Buffer;
import org.apache.commons.collections.Predicate;
import org.apache.commons.collections.Transformer;
import org.apache.commons.collections.buffer.BlockingBuffer;
import org.apache.commons.collections.buffer.PredicatedBuffer;
import org.apache.commons.collections.buffer.TransformedBuffer;
import org.apache.commons.collections.buffer.UnboundedFifoBuffer;

/**
 * An approach to converting strings to numbers using the ASF Commons
 * Collections buffers.
 *
 * @author Liviu Tudor http://about.me/liviutudor
 */
public class ConvertStringBuffers {
    /**
     * Program entry point.
     *
     * @param arg
     *            Command-line arguments. Ignored.
     */
    public static void main(String[] args) {
        /* Ensure we only accepts Integer. */
        Predicate predicate = new Predicate() {
            @Override
            public boolean evaluate(Object object) {
                if (object == null) {
                    return false;
                }
                if (!(object instanceof Integer)) {
                    return false;
                }
                return true;
            }
        };
        /* Used to convert string to int. */
        Transformer transformer = new Transformer() {
            @Override
            public Object transform(Object input) {
                try {
                    if (!(input instanceof String)) {
                        return 0;
                    }
                    String sInput = (String) input;
                    return Integer.valueOf(sInput);
                } catch (Exception e) {
                    return 0;
                }
            }
        };
        Buffer buffer = TransformedBuffer.decorate(
                PredicatedBuffer.decorate(BlockingBuffer.decorate(new UnboundedFifoBuffer()), predicate),
                transformer);
        StringProducer producer = new StringProducer(buffer);
        StringConsumer consumer = new StringConsumer(buffer);
        producer.addEndListener(consumer);

        final CountDownLatch latch = new CountDownLatch(1);
        EndListener listener = new EndListener() {
            @Override
            public void end() {
                latch.countDown();
            }
        };
        producer.addEndListener(listener);

        producer.startProduce();
        try {
            latch.await();
        } catch (InterruptedException e) {
            System.exit(1);
        }
    }
}
