package convert.buffers;

import org.apache.commons.collections.Buffer;

/**
 * Simple consumer. Takes a string from a given "buffer" and transforms it into
 * an integer and then prints it on console. The transformation is done by
 * building a TransformedBuffer on top of the given one.
 *
 * @author Liviu Tudor http://about.me/liviutudor
 */
public class StringConsumer implements EndListener {
    /** Buffer used to consume data from. */
    private Buffer buffer;

    /** Consumption of strings is done in this separate thread. */
    private Thread consumerThread;

    /**
     * Creates a consumer which starts right away consuming items from the
     * queue. The {@link #consumerThread consumer thread} runs until
     * {@link #end() the end notification} is received.
     *
     * @param queue
     *            Queue to consume from
     * @param transformer
     *            Implementation of an operation which converts strings to
     *            integers
     */
    public StringConsumer(Buffer buffer) {
        this.buffer = buffer;
        this.consumerThread = new Thread(new Runnable() {
            @Override
            public void run() {
                while (!StringConsumer.this.consumerThread.isInterrupted()) {
                    try {
                        Integer i = (Integer) StringConsumer.this.buffer.remove();
                        System.out.println(i);
                    } catch (Exception e) {
                        break;
                    }
                }
            }
        });
        this.consumerThread.start();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void end() {
        consumerThread.interrupt();
    }
}
