package convert.oop;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.SynchronousQueue;

/**
 * A more OOP-centric approach to converting strings to numbers.
 *
 * @author Liviu Tudor http://about.me/liviutudor
 */
public class ConvertStringOop {
    /**
     * Program entry point.
     *
     * @param arg
     *            Command-line arguments. Ignored.
     */
    public static void main(String[] args) {
        BlockingQueue<String> queue = new SynchronousQueue<String>();
        Transformer<String, Integer> transformer = new StringToIntTransformer();
        StringProducer producer = new StringProducer(queue);
        StringConsumer consumer = new StringConsumer(queue, transformer);
        producer.addEndListener(consumer);

        final CountDownLatch latch = new CountDownLatch(1);
        EndListener listener = new EndListener() {
            @Override
            public void end() {
                latch.countDown();
            }
        };
        producer.addEndListener(listener);

        producer.startProduce();
        try {
            latch.await();
        } catch (InterruptedException e) {
            System.exit(1);
        }
    }
}
